//
//  FUNCTION : STRUCT.PRG
//
//   PURPOSE : Create a list of all databases located in the folder
//             Showing the complete structure of the fields in each
//             database.  Along with the TAG names and list of the
//             condition that created the tag.  Sorry no NTX index
//             support.
//
//       NEW : Added Pre scan of database's and indexes to create a page
//             control that minimizes a structure from printed on two pages
//
//             Test for length of Index condition and split the line if the
//             length is longer then the page width.
//
//             A TITLE for the DATABASES can be added when the program is
//             started.
//
//    OUTPUT : Selectable .TXT - Located in the folder of the databases.
//
//    UPDATE :  6 OCT 2010 - ORIGINAL DATE 7 SEP 2009
//
// COPYRIGHT : John P. Hohensee - Released to Public domain
//             ZAC Micro Systems
//             jhohensee@zacmicro.com
//-----------------------------------------------------------------------------

#include "DCDIALOG.CH"
#include "set.ch"
#include "xbp.ch"
#include "inkey.ch"
#include "appevent.ch"
#include "DCAPP.CH"
#include "DCBITMAP.CH"
#include "DCPRINT.CH"
#include "common.ch"
#include "cdxdbe.ch"
#include "dbfdbe.ch"
#include "sdfdbe.ch"
#include "foxdbe.ch"
#include "directry.ch"
#include "dbstruct.ch"

static asTags   // Contains the INDEX TAG Names and Index Conditions
static nsRow    // Contains the current Printing row
static nsPg     // Contains the current Page number

#pragma library( "DCLIPX.LIB" )
#pragma library( "DCLIP1.LIB" )

#define CRLF ( chr(13)+chr(10) )

//-----------------------------------------------------------------------------
// FUNCTION : MAIN()
//  PURPOSE : Begin database structure listing
//   CALLED : NONE
//    CALLS : _LoadDbes(),  _Struct()
//   IMPORT : NONE
//   EXPORT : NONE
//-----------------------------------------------------------------------------

procedure Main()

  _LoadDbes()
  _Struct()
  return


//-----------------------------------------------------------------------------
// FUNCTION : _STRUCT()
//  PURPOSE : Build list of database in current directory
//   CALLED : _Struct()
//    CALLS : _TestDbf(),  _ShowStruct(), _BuildIdx()
//   IMPORT : NONE
//   EXPORT : NONE
//   UPDATE : 17 OCT 2010
//-----------------------------------------------------------------------------

static procedure _Struct()

  local getlist[0], GetOptions, lOk := .F., cFont := "10.Courier New", i, j
  local cTitle := space(40), aDir := directory('*.DBF'), cDbfName, cDBF, cLine
  local nTitle, nOffSet, nOvr, nFldCnt, aList, nOver, cName, lNewPage, oBar
  local cFirst, cLast, k, nPos, nPos2, cPrntLocn, cBgn, cDirLocn, cPrintDest
  local cPrintName := "DATABASE", cPath := CurDrive()+":\"+CurDir(CurDrive())
  local cTxtPath := CurDrive()+":\"+CurDir( CurDrive() )

  store ""        to cDbfName, cDBF,  cLine
  store 0         to nTitle,   nsRow, nsPg, nOffSet
  store space(60) to cDirLocn, cPrintDest                     // 17 OCT 2010
  //
  aList := {}
  //
  @02,05 DCSAY "PRINT STRUCTURE TITLE"  SIZE 30
  @02,50 DCSAY "VERSION - 16 OCT 2010"  SIZE 30
  @03,05 DCGET cTitle   PICTURE repl("!",40)
  //
  @05,05 DCSAY "STARTING AT @ " + trim( cPath ) SIZE 60 COLOR 2 // 25 SEP 2010
  @06,05 DCGET cDirLocn  POPUP {|c| DC_PopDir("", "CD",,"*.") } // 17 OCT 2010
  //
  @08,05 DCSAY "OUTPUT NAME"            SIZE 14
  @08,19 DCGET cPrintName PICTURE "@K!!!!!!!!"                  // 17 OCT 2010
  //
  @10,05 DCSAY "PUT THE TEXT HERE"      SIZE 21
  @11,05 DCGET cPrintDest  POPUP {|c| DC_PopDir("","CD",,"*.") }//17 OCT 2010
  //
  @13,02 DCTOOLBAR oBar SIZE 18,1.2 BUTTONSIZE 9,1.2
  //
  DCSETPARENT TO oBar
  DCADDBUTTONXP CAPTION "ACCEPT"    COLOR 1,5   OUTLINE BORDERCOLOR 1         ;
      ACTION {|| lOk := .T., DC_ReadGuiEvent(DCGUI_EXIT_OK,    getlist) }
  DCADDBUTTONXP CAPTION "ABORT"     COLOR 1,3   OUTLINE BORDERCOLOR 1         ;
      ACTION {|| lOk := .F., DC_ReadGuiEvent(DCGUI_EXIT_ABORT, getlist) }
  DCSETPARENT TO
  //
  DCGETOPTIONS WINDOWWIDTH 580 WINDOWHEIGHT 340                               ;
      NOMINBUTTON NOMAXBUTTON NORESIZE SAYFONT cFont GETFONT cFont TABSTOP
  //
  DCREAD GUI TITLE "PRINT STRUCTURES" OPTIONS GetOptions MODAL TO lOk         ;
      EVAL {|o| setappwindow( o ) }
  //
  if !lOk
    quit
  endif
  //
  cPrintName  := alltrim( cPrintName )                          // 17 SEP 2010
  cPath       := alltrim( upper( cDirLocn ) ) + "\"             // 17 OCT 2010
  cPrintDest  := alltrim( upper( cPrintDest ) ) + "\"           // 17 OCT 2010
  //
  DC_ChDir( cPath )                                             // 25 SEP 2010
  DC_SetDefault( cPath )                                        // 25 SEP 2010
  aDir      := directory( "*.DBF" )                             // 25 SEP 2010
  //-
  //-- ALIGN TITLE ON PAGE
  //-
  if !empty( cTitle )
    cTitle  := alltrim( cTitle )
    nTitle  := ( len( cTitle ) / 2 )
  else
    cTitle  := "DATABASE STRUCTURES"
    nTitle  := ( len( cTitle ) / 2 )
  endif
  nsRow     := 1
  //-
  //-- ERASE DESTINATION FILE NAME [ OLD AND NEW ]
  //-
  ferase( ( cPrintDest ) + "DATABASE.TXT" )                     // 17 OCT 2010
  ferase( ( cPrintDest + cPrintName ) + ".TXT" )                // 17 OCT 2010
  //
  set device  to PRINT
  set printer to ( cPrintDest + cPrintName ) + ".TXT"           // 17 OCT 2010
  //
  DCMSGBOX "PRINTING TO :"+cPrintDest+cPrintName+".TXT:  FROM:"+cPath         ;
      TIMEOUT 4
  asort(aDir,,, {|a,b| upper(a[1]) < upper(b[1])})    // SORT DBF NAMES
  //-
  //-- SCAN FOR LENGTHS OF DATABASE AND INDEX
  //-
  for i := 1 to len( aDir )         // FOR NUMBER OF FILES FOUND
    cName   := aDir[i, F_NAME]      //  cName IS THE DATABASE NAME
    use (cName) alias "TEST" new    //  OPEN THE DATABASE
    select( "TEST" )                //  SELECT THE DATABASE
    nFldCnt := fcount()             //  GET FIELD COUNT
    nOver   := ( nFldCnt % 4 )      //  Number of Rows NOT Full
    nsRow   := int( nFldCnt / 4 )   //  Number of Rows Full required
    if nOver > 0                    //  IF NO FULL GT ZERO
      nsRow++                       //   ADD ONE ROW
    endif                           //  ENDIF
    nsRow   += _CkIdx( cName )      //  ADD ROWS FOR IDX
    nsRow++                         //  ADD BLANK LINE
    dbclosearea( "TEST" )           //  CLOSE THE DATABASE
    aadd( aList, { nsRow } )        //  NUMBER OF ROWS FOR EACH DATABASE
  next                              // NEXT
  //-
  //- NOW RUN FOR PRINTOUT
  //-
  for i := 1 to len( aDir )         // FOR 1 TO LEN( aDir )
    //                              //
    if ( nsPg == 0 ) .or. ( aList[i, 1] + nsRow ) > 50
      if nsPg # 0
        eject                       //    NEXT PAGE - FORM FEED
        @00,00 say CRLF             //    BLANK LINE
      endif
      nsPg++                        //   INCREASE PAGE NUMBER
      @01,10 say "[ " + cTitle + " ]"                           // 17 OCT 2010
      @01,70 say "PAGE " + str( nsPg, 2 )
      nsRow := 3
    endif                           //  ENDIF
    //                              //
    _TestDbf(    aDir[ i, F_NAME ] )//  DETERMINE IF DBFNTX OR FOXDBX
    _ShowStruct( aDir[ i, F_NAME ] )//  PRINT FIELD DEF FOUR TO A LINE
    _BuildIdx(   aDir[ i, F_NAME ] )//  BUILD LIST OF TAGS USED TO asTag
    //                              //
    if len( asTags ) > 0            //  IF asTag HAS ANY DATA
      @nsRow,05 say "IDX:"          //   PRINT "IDX:"
      //                            //
      //          111111111122222222223
      //012345678901234567890123456789
      //     IDX: .TAG.   .DISCRIPTION OF TAG
      //                            //
      for j := 1 to len( asTags )   //   FOR J TO NUMBER OF IDX TAGS
        cLine := alltrim( asTags[ j ] )// cLine IS TAG NAME and CONDITION
        if len( cLine ) > 72        //    IF cLine LENGTH EXCEEDS 72
          nPos    := len( cLine )   //     SPLIT THE LINE INTO TWO PARTS
          nPos2   := nPos / 2       //
          cFirst  := left( cLine, nPos2 )
          cLast   := right( cLine, nPos - nPos2)
          @nsRow++,10 say cFirst + "  ;"// @ROW,10 SAY FIRST 72 CHAR
          @nsRow++,25 say cLast     //     @ROW,25 SAY LAST CHAR
        else                        //    ELSE
          @nsRow++,10 say cLine     //     @ROW,10 SAY TAGS FIT ON ONE LINE
        endif                       //    ENDIF
      next                          //   NEXT J
      //                            //
    endif                           //  ENDIF len( asTags ) > ZERO
    //                              //
    nsRow++                         //  ADD ONE TO ROW
    //                              //
  next                              // NEXT I
  //                                //
  eject                             // EJECT - FORM FEED
  set printer to                    //
  set device  to SCREEN             //
  DCMSGBOX "DONE: DBF's "+str(len(aDir),4)+" Compliments of ZAC Micro Systems";
      TIMEOUT 3
  DC_ChDir( cBgn )                  //
  return                            //


//-----------------------------------------------------------------------------
// FUNCTION : _CKIDX()
//  PURPOSE : Determine the number of TAGS that will be printed
//   CALLED : _Struct()
//    CALLS : NONE
//   IMPORT : cDbfName
//   EXPORT : NONE
//   UPDATE :  6 OCT 2010
//-----------------------------------------------------------------------------

static function _CkIdx( cName )

  local cTagName, cIdxName, cKey, nSel := 1
  local i := at( ".", cName ), nCnt := 0

  if i > 0
    cName := upper( substr( cName, 1, i - 1 ) )
  endif
  cTagName := ""
  cIdxName := alltrim( cName ) + ".CDX"
  //
  if !empty( cIdxName ) .and. file( cIdxName )
    use ( cName ) index ( cIdxName ) alias "TEST"
    //
    cTagName := ordkey( nSel++ )    // START WITH ordkey( 2 )
    while !empty( cTagName )        // WHILE WE HAVE A TAG NAME
      nCnt++                        //  INCREASE COUNT BY ONE
      cTagName := ordkey( nSel++ )  //  NOW CHECK ordkey( 3 )
      if len( cTagName ) > 72       //  IF TAGNAME GREATER THEN 72
        nCnt++                      //   INCREASE COUNT BY ONE
      endif                         //  ENDIF
    end                             // END
  endif                             //
  nCnt++                            // ALWAY ADD ONE LINE BETWEEN DATABASES
  return nCnt


//-----------------------------------------------------------------------------
// FUNCTION : _SHOWSTRUCT()
//  PURPOSE : Show database structure list for given database
//   CALLED : _Struct()
//    CALLS : NONE
//   IMPORT : cDbfName
//   EXPORT : NONE
//   UPDATE : 11 SEP 2010
//-----------------------------------------------------------------------------

static procedure _ShowStruct( cDbfName )

  local aStruct, i := at('.', cDbfName ), lTitle := .F., cLine
  local cName := upper( substr( cDbfName, 1, i - 1 ) )

  use ( cName ) alias ( "TEST" ) new
  aStruct := TEST->( dbstruct() )
  for i := 1 to len( aStruct )
    if !lTitle
      @nsRow++,10 say "=== DBF : " + padr( alltrim( cName ), 12) + " :==="
      lTitle := .T.
    endif
    cLine := padr( aStruct[i,1] + ",",11) +       ; // NAME
                   aStruct[i, 2] + "," +          ; // TYPE
              str( aStruct[i,3],2) + "," +        ; // LEN
              alltrim(str(aStruct[i,4],2) )         // DEC
    //-
    //-- ( 1 % 4) = 1 | ( 2 % 4) = 2 | ( 3 % 4) = 3 | ( 4 % 4) = 0
    //-- ( 5 % 4) = 1 | ( 6 % 4) = 2 | ( 7 % 4) = 3 | ( 8 % 4) = 0
    //-- ( 9 % 4) = 1 | (10 % 4) = 2 | (11 % 4) = 3 | (12 % 4) = 0
    //-- (13 % 4) = 1 | (14 % 4) = 2 | (15 % 4) = 3 | (16 % 4) = 0
    //--
    //          11111111112222222222333333333344444444445555555555666666666677777777778
    //012345678901234567890123456789012345678901234567890123456789012345678901234567890
    //     .08.....,02,.04.  .08.....,02,.04.  .08.....,02,.04.  .08.....,02,.04.
    //       FIRST COLUMN     SECOND COLUMN      THIRD COLUMN      FORTH COLUMN
    //
    do case
      case ( i % 4 ) == 1
        @nsRow,05 say cLine
      case ( i % 4 ) == 2
        @nsRow,23 say cLine
      case ( i % 4 ) == 3
        @nsRow,41 say cLine
      case ( i % 4 ) == 0
        @nsRow++,59 say cLine
    endcase
  next
  //
  if ( i % 4 ) # 1
    nsRow++
  endif
  dbclosearea( 'TEST' )
  return


//-----------------------------------------------------------------------------
// FUNCTION : _BUILDIDX()
//  PURPOSE : Build list of INDEXES used and their keys
//   CALLED : _Struct()
//    CALLS : NONE
//   IMPORT : cDbfName
//   EXPORT : NONE
//   UPDATE : 11 JUL 2009
//-----------------------------------------------------------------------------

static procedure _BuildIdx( cDbfName )

  local cTagName, cIdxName, cMemName, nSel := 1, cDBF
  local i := at('.', cDbfName ), cKey

  if i > 0
    cDBF   := upper( substr( cDbfName, 1, i - 1 ) )
  else
    cDBF   := cDbfName
  endif
  asTags   := {}
  cTagName := ""
  cIdxName := alltrim( cDBF ) + ".CDX"
  //
  if !empty( cIdxName) .and. file( cIdxName )
    //-
    //-- HAVE DBF AND INDEX
    //-
    use ( cDBF ) index ( cDBF ) alias ( cDBF ) exclusive
    //-
    //-- Find CDX Tags for index
    //-
    cTagName := ordkey( nSel++ )
    while !empty( cTagName )
      //-
      //-- build index key
      //-
      ordsetfocus( nSel - 1 )
      cKey := padr( ordname( nSel - 1 ), 10) + " : " + ordkey( nSel - 1 )
      if ordisdescend()
        cKey += " +DES"
      endif
      if ordisunique()
        cKey += " +UNQ"
      endif
      aadd( asTags, cKey )
      cTagName := ordkey( nSel++ )
    end
    ( cDBF )->( dbclosearea() )
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _TESTDBF()
//  PURPOSE : Check for MEMO file to determine the dbe to be used
//   CALLED : _Struct()
//    CALLS : NONE
//   IMPORT : cDbfName
//   EXPORT : TRUE
//   UPDATE :  7 SEP 2010
//-----------------------------------------------------------------------------

static procedure _TestDbf( cDbfName )

  local i := at('.', cDbfName ), cName

  cName := substr( cDbfName, 1, i - 1 )
  if file( cName + ".DBT" )
    dbesetdefault( "DBFNTX" )
  else
    dbesetdefault( "FOXCDX" )
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _LOADDBES()
//  PURPOSE : Load dbe engines used and unused
//   CALLED : _Struct()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : NONE
//   UPDATE :  7 SEP 2009
//-----------------------------------------------------------------------------

static procedure _LoadDbes()

  local i, aDbeList := DbeList(), cDbeList := '', aDbe := {}

  if valtype(aDbeList) = 'A'
    for i := 1 to len(aDbeList)
      cDbeList += aDbeList[i,1] + ','
    next
  endif
  aDbeList := cDbeList
  //-
  //-- Standard CLIPPER - NTX
  //-
  if !('DBFDBE' $ aDbeList) .and. !DbeLoad( "DBFDBE",.T.)
    alert( "Database-Engine DBFDBE not loaded" )
  endif
  if !('NTXDBE' $ aDbeList) .and. !DbeLoad( "NTXDBE",.T.)
    alert( "Database-Engine NTXDBE not loaded"  )
  endif
  if !('DBFNTX' $ aDbeList) .and. !DbeBuild( "DBFNTX", "DBFDBE", "NTXDBE" )
    alert( "DBFNTX Database-Engine, Could not build engine" )
  else
    aadd(aDbe, {"DBFNTX"})
  endif
  //-
  //-- Standard FoxPro 2.6 - CDX
  //-
  if !('CDXDBE' $ aDbeList) .and. !DbeLoad( "CDXDBE",.T.)
    alert( "Database-Engine CDXDBE not loaded" )
  endif
  if !('DBFCDX' $ aDbeList) .and. !DbeBuild( "DBFCDX", "DBFDBE", "CDXDBE" )
    alert( "DBFCDX Database-Engine, Could not build engine" )
  else
    aadd(aDbe, {"DBFCDX"})
  endif
  //-
  //-- Standard FoxPro 3.0 - CDX
  //-
  if !('FOXDBE' $ aDbeList) .and. !DbeLoad( "FOXDBE",.T.)
    alert( "Database-Engine FOXDBE not loaded" )
  endif
  if !('FOXCDX' $ aDbeList) .and. !DbeBuild( "FOXCDX", "FOXDBE", "CDXDBE" )
    alert( "FOXCDX Database-Engine, Could not build engine" )
  else
    aadd(aDbe, {"FOXCDX"})
  endif
  return


